@@ -3,9 +3,13 @@ Start at the infrastructure for OStatus, including webfinger,
 user profiles, etc.
 """
 
-from tiddlyweb.control import filter_tiddlers
+import urllib
+import urllib2
+
 from tiddlyweb.store import StoreError, NoUserError
+from tiddlyweb.model.bag import Bag
 from tiddlyweb.model.tiddler import Tiddler
+from tiddlyweb.model.policy import ForbiddenError, UserRequiredError
 from tiddlyweb.model.user import User
 from tiddlyweb.web.handler.search import get as search
 from tiddlyweb.web.http import HTTP404, HTTP400, HTTP415
@@ -14,6 +18,8 @@ from tiddlyweb.web.util import (server_host_url, server_base_url,
 from tiddlywebplugins.tiddlyspace.spaces import space_uri
 from tiddlywebplugins.tiddlyspace.web import determine_host
 from tiddlyweb.wikitext import render_wikitext
+from tiddlywebplugins.dispatcher.listener import Listener as BaseListener
+from tiddlywebplugins.utils import get_store
 
 
 HOST_META_TEMPLATE = """<?xml version="1.0" encoding="UTF-8"?>
@@ -37,7 +43,7 @@ WEBFINGER_TEMPLATE = """<?xml version="1.0"?>
       href="%(server_host)s/profiles/%(username)s"
       type="text/html"/>
 <Link rel="http://schemas.google.com/g/2010#updates-from"
-      href="%(server_host)s/profiles/%(username)s"
+      href="%(server_host)s/profiles/%(username)s.atom"
       type="application/atom+xml"/>
 </XRD>
 """
@@ -100,6 +106,7 @@ def html_profile(environ, start_response):
     modified tiddlers.
     """
     username = environ['wsgiorg.routing_args'][1]['username']
+    usersign = environ['tiddlyweb.usersign']
 
     store = environ['tiddlyweb.store']
 
@@ -108,8 +115,7 @@ def html_profile(environ, start_response):
     except NoUserError:
         raise HTTP404('Profile not found for %s' % username)
 
-    activity_feed = (server_host_url(environ) +
-            '/profiles/%s.atom' % encode_name(username))
+    activity_feed = profile_atom_url(environ, username)
 
     environ['tiddlyweb.links'] = [
            '<link rel="alternate" type="application/atom+xml"'
@@ -126,10 +132,25 @@ def html_profile(environ, start_response):
 
     tiddlers = store.search(_search_string(username))
     tiddlers_list = []
-    for tiddler in filter_tiddlers(tiddlers, 'sort=-modified;limit=20'):
-        tiddlers_list.append('<li><a href="/bags/%s/tiddlers/%s">%s</a></li>'
-                % (encode_name(tiddler.bag), encode_name(tiddler.title),
-                    tiddler.title))
+
+    # XXX this bag_readable loop is copied from
+    # tiddlyweb.web.handler.search:get, it should be factored
+    # out of there for resue
+    bag_readable = {}
+    for tiddler in tiddlers:
+        try:
+            if bag_readable[tiddler.bag]:
+                tiddlers_list.append(_tiddler_in_list(tiddler))
+        except KeyError:
+            bag = Bag(tiddler.bag)
+            bag = store.get(bag)
+            try:
+                bag.policy.allows(usersign, 'read')
+                tiddlers_list.append(_tiddler_in_list(tiddler))
+                bag_readable[tiddler.bag] = True
+            except(ForbiddenError, UserRequiredError):
+                bag_readable[tiddler.bag] = False
+
     profile_tiddlers = '\n'.join(tiddlers_list)
 
     avatar_path = '/recipes/%s_public/tiddlers/SiteIcon' % username
@@ -159,6 +180,11 @@ def host_meta(environ, start_response):
         server_base_url(environ)}]
 
 
+def profile_atom_url(environ, username):
+    return (server_host_url(environ) +
+            '/profiles/%s.atom' % encode_name(username))
+
+
 def webfinger(environ, start_response):
     """
     Display the webfinger information for a given user.
@@ -183,9 +209,64 @@ def webfinger(environ, start_response):
         'host': http_host, 'server_host': server_base_url(environ)}]
 
 
+class Listener(BaseListener):
+
+    TUBE = 'pushping'
+    STORE = None
+
+    def _act(self, job):
+        if not self.STORE:
+            self.STORE = get_store(self.config)
+        info = self._unpack(job)
+
+        tiddler = Tiddler(info['tiddler'], info['bag'])
+        try:
+            self.STORE.get(tiddler)
+        except StoreError:
+            return None #  Tiddler's not there, no need to notify
+        user = tiddler.modifier
+        if self._user_has_profile(user):
+            self._send_ping(user)
+
+    def _user_has_profile(self, user):
+        try:
+            tiddler = Tiddler('profile', '%s_public' % user)
+            self.STORE.get(tiddler)
+        except StoreError:
+            return False
+        return True
+
+    def _send_ping(self, user):
+        data = {
+                'hub.mode': 'publish',
+                'hub.url': profile_atom_url({'tiddlyweb.config': self.config},
+                    user),
+                }
+
+        try:
+            target = self.config['atom.hub']
+        except KeyError:
+            return None
+        encoded_data = urllib.urlencode(data)
+
+        try:
+            response = urllib2.urlopen(target, encoded_data)
+            status = response.getcode()
+            if status != '204':
+                logging.warn('non 204 response from hub: %s', status)
+        except urllib2.URLError, exc:
+            logging.warn('error when publishing to hub: %s', exc)
+
+
 def _search_string(username):
     """
     Construct the search string to be used for creating
     the recent tiddlers for this username.
     """
-    return 'modifier:%s' % username
+    return 'modifier:%s _limit:20' % username
+
+
+def _tiddler_in_list(tiddler):
+    return ('<li><a href="/bags/%s/tiddlers/%s">%s</a></li>'
+            % (encode_name(tiddler.bag), encode_name(tiddler.title),
+                tiddler.title))
